package java.nio.channels;

import java.nio.ByteBuffer;
import java.net.DatagramSocket;
import java.net.SocketOption;
import java.net.SocketAddress;
import java.net.ProtocolFamily;
import java.io.IOException;
import java.nio.channels.spi.SelectorProvider;
import java.nio.channels.spi.AbstractSelectableChannel;

public abstract class DatagramChannel extends AbstractSelectableChannel implements ByteChannel, ScatteringByteChannel, GatheringByteChannel, MulticastChannel
{
    protected DatagramChannel(SelectorProvider provider) {
        super(provider);
    }
    
    public static DatagramChannel open() throws IOException {
        return SelectorProvider.provider().openDatagramChannel();
    }
    
    public static DatagramChannel open(ProtocolFamily protocolFamily) throws IOException {
        return SelectorProvider.provider().openDatagramChannel(protocolFamily);
    }
    
    @Override
    public final int validOps() {
        return 5;
    }
    
    @Override
    public abstract DatagramChannel bind(SocketAddress p0) throws IOException;
    
    @Override
    public abstract <T> DatagramChannel setOption(SocketOption<T> p0, T p1) throws IOException;
    
    public abstract DatagramSocket socket();
    
    public abstract boolean isConnected();
    
    public abstract DatagramChannel connect(SocketAddress p0) throws IOException;
    
    public abstract DatagramChannel disconnect() throws IOException;
    
    public abstract SocketAddress getRemoteAddress() throws IOException;
    
    public abstract SocketAddress receive(ByteBuffer p0) throws IOException;
    
    public abstract int send(ByteBuffer p0, SocketAddress p1) throws IOException;
    
    @Override
    public abstract int read(ByteBuffer p0) throws IOException;
    
    @Override
    public abstract long read(ByteBuffer[] p0, int p1, int p2) throws IOException;
    
    @Override
    public final long read(ByteBuffer[] array) throws IOException {
        return this.read(array, 0, array.length);
    }
    
    @Override
    public abstract int write(ByteBuffer p0) throws IOException;
    
    @Override
    public abstract long write(ByteBuffer[] p0, int p1, int p2) throws IOException;
    
    @Override
    public final long write(ByteBuffer[] array) throws IOException {
        return this.write(array, 0, array.length);
    }
    
    @Override
    public abstract SocketAddress getLocalAddress() throws IOException;
}
